"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$expression = void 0;
const pierrejs_1 = require("pierrejs");
const literal_1 = require("./literal");
const utils_1 = require("./utils");
/* $lab:coverage:off$ */
var State;
(function (State) {
    State[State["_START"] = 0] = "_START";
    State[State["GOT_LITERAL"] = 1] = "GOT_LITERAL";
    State[State["GOT_PATH"] = 2] = "GOT_PATH";
})(State || (State = {}));
/* $lab:coverage:on$ */
const topOfStack = (stack) => stack[stack.length - 1];
const peekEnd = pierrejs_1.default.lookAhead(pierrejs_1.default.string('}}'));
const path = pierrejs_1.default.regex('context path', /^[a-zA-Z0-9\-_$\.]+/).map(path => ({
    type: 'EXPRESSION',
    path,
    params: [],
}));
exports.$expression = pierrejs_1.default.context('expression', function* () {
    const stack = [[]];
    let state = State._START;
    const expressionFromStack = expr => {
        if (!Array.isArray(expr)) {
            return expr;
        }
        const [stmt, ...params] = expr;
        if (stmt.type === 'LITERAL') {
            return stmt;
        }
        stmt.params = params.map(expressionFromStack);
        return stmt;
    };
    /* $lab:coverage:off$ */
    while (true) {
        /* $lab:coverage:on$ */
        switch (state) {
            case State._START: {
                yield utils_1.optionalSpaces;
                const l = yield pierrejs_1.default.optional(literal_1.$literal);
                if (l) {
                    topOfStack(stack).push(l);
                    state = State.GOT_LITERAL;
                    break;
                }
                const p = yield pierrejs_1.default.optional(path);
                if (p) {
                    topOfStack(stack).push(p);
                    state = State.GOT_PATH;
                    break;
                }
                yield pierrejs_1.default.fail('Expected literal, helper or context path');
            }
            case State.GOT_LITERAL: {
                yield utils_1.optionalSpaces;
                const end = yield pierrejs_1.default.optional(peekEnd);
                if (end) {
                    if (stack.length > 1) {
                        yield pierrejs_1.default.fail('Expected ")", make sure every parenthesis was closed');
                    }
                    return expressionFromStack(stack[0]);
                }
                const subExprEnd = yield pierrejs_1.default.optional(pierrejs_1.default.string(')'));
                if (subExprEnd) {
                    if (stack.length <= 1) {
                        yield pierrejs_1.default.fail('Unexpected ")", this parenthesis wasn\'t opened');
                    }
                    const expr = stack.pop();
                    topOfStack(stack).push(expr);
                    state = State.GOT_PATH;
                    break;
                }
                yield pierrejs_1.default.fail('Expected "}}". Literal mustaches cannot have parameters.');
            }
            case State.GOT_PATH: {
                yield utils_1.optionalSpaces;
                const end = yield pierrejs_1.default.optional(peekEnd);
                if (end) {
                    if (stack.length > 1) {
                        yield pierrejs_1.default.fail('Expected ")", make sure every parenthesis was closed');
                    }
                    return expressionFromStack(stack[0]);
                }
                const param = yield pierrejs_1.default.optional(pierrejs_1.default.either(literal_1.$literal, path));
                if (param) {
                    topOfStack(stack).push(param);
                    state = State.GOT_PATH;
                    break;
                }
                const subExpr = yield pierrejs_1.default.optional(pierrejs_1.default.string('('));
                if (subExpr) {
                    stack.push([]);
                    state = State._START;
                    break;
                }
                const subExprEnd = yield pierrejs_1.default.optional(pierrejs_1.default.string(')'));
                if (subExprEnd) {
                    if (stack.length <= 1) {
                        yield pierrejs_1.default.fail('Unexpected ")", this parenthesis wasn\'t opened');
                    }
                    const expr = stack.pop();
                    topOfStack(stack).push(expr);
                    state = State.GOT_PATH;
                    break;
                }
                yield pierrejs_1.default.fail('Expected expression parameters or "}}"');
            }
            /* $lab:coverage:off$ */
            default:
                yield pierrejs_1.default.fail(`Unexpected state ${state} at expression parser`);
            /* $lab:coverage:on$ */
        }
    }
});
