"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayHelpers = void 0;
const utils_1 = require("../../utils");
function first(arr) {
    if (!Array.isArray(arr)) {
        return null;
    }
    return arr[0] || null;
}
function last(arr) {
    if (!Array.isArray(arr)) {
        return null;
    }
    return arr[arr.length - 1] || null;
}
function itemAt(arr, index) {
    const idx = Number(index);
    (0, utils_1.ensure)(!isNaN(idx), 'itemAt expects second argument to be a number');
    if (!Array.isArray(arr)) {
        return null;
    }
    return arr[idx] || null;
}
function length(arr) {
    if (!Array.isArray(arr) && typeof arr !== 'string') {
        return -1;
    }
    return arr.length;
}
function after(arr, index) {
    const idx = Number(index);
    (0, utils_1.ensure)(!isNaN(idx), 'after expects second argument to be a number');
    if (!Array.isArray(arr)) {
        return [];
    }
    return arr.slice(idx);
}
function before(arr, index) {
    const idx = Number(index);
    (0, utils_1.ensure)(!isNaN(idx), 'before expects second argument to be a number');
    if (!Array.isArray(arr)) {
        return [];
    }
    return arr.slice(0, idx);
}
function slice(arr, start, end) {
    (0, utils_1.ensure)(typeof start === 'number' || typeof start === 'undefined', 'slice expects second argument to be a number');
    (0, utils_1.ensure)(typeof end === 'number' || typeof end === 'undefined', 'slice expects third argument to be a number');
    if (!Array.isArray(arr)) {
        return [];
    }
    return arr.slice(start, end);
}
function includes(arr, value) {
    if (typeof arr === 'number') {
        return String(arr).includes(value);
    }
    if (!Array.isArray(arr) && typeof arr !== 'string') {
        return false;
    }
    return arr.includes(value);
}
function isArray(arr) {
    return Array.isArray(arr);
}
function each(arr) {
    if (Array.isArray(arr)) {
        return arr;
    }
    return [arr];
}
function forEach(arr) {
    const original = Array.isArray(arr) ? arr : [arr];
    const result = original.map((item, index) => {
        const obj = Object.create(null);
        obj.item = item;
        obj.index = index;
        obj.total = original.length;
        obj.isFirst = index === 0;
        obj.isLast = index === original.length - 1;
        return obj;
    });
    return result;
}
function join(arr, separator) {
    if (!Array.isArray(arr)) {
        return '';
    }
    return arr.join(String(separator));
}
function merge(...arrs) {
    return arrs.reduce((acc, arr) => {
        if (Array.isArray(arr)) {
            return [...acc, ...arr];
        }
        return [...acc, arr];
    }, []);
}
function reverse(arr) {
    if (Array.isArray(arr)) {
        return arr.reverse();
    }
    if (typeof arr === 'string') {
        return arr.split('').reverse().join('');
    }
    return [];
}
function pluck(arr, key) {
    if (!Array.isArray(arr)) {
        return [];
    }
    if (utils_1.UNSAFE_KEYS.has(key)) {
        return [];
    }
    return arr
        .map(item => item[key])
        .filter(item => item !== void 0);
}
function unique(arr) {
    if (!Array.isArray(arr)) {
        return [];
    }
    return Array.from(new Set(arr));
}
function isEmpty(arr) {
    if (!Array.isArray(arr)) {
        return false;
    }
    return arr.length === 0;
}
function splice(arr, start, deleteCount) {
    (0, utils_1.ensure)(typeof start === 'number', 'splice expects first argument to be a number');
    (0, utils_1.ensure)(typeof deleteCount === 'number' || typeof deleteCount === 'undefined', 'splice expects second argument to be a number');
    if (!Array.isArray(arr)) {
        return [];
    }
    return arr.splice(start);
}
function compareNumbersAscending(a, b) {
    return a - b;
}
function compareNumbersDescending(a, b) {
    return b - a;
}
function compareStrings(a, b) {
    return a.localeCompare(b);
}
function allElementsAreNumeric(arr) {
    return arr.every((value) => typeof value === 'number');
}
function allElementsAreString(arr) {
    return arr.every((value) => typeof value === 'string');
}
function sort(arr, desc) {
    (0, utils_1.ensure)(Array.isArray(arr), 'sort expects an array of numbers or strings');
    (0, utils_1.ensure)(allElementsAreNumeric(arr) || allElementsAreString(arr), 'sort expects all elements to be either string or numeric');
    if (desc) {
        return typeof arr[0] === 'number' ? arr.sort(compareNumbersDescending) : arr.sort(compareStrings).reverse();
    }
    return typeof arr[0] === 'number' ? arr.sort(compareNumbersAscending) : arr.sort(compareStrings);
}
exports.arrayHelpers = Object.assign(Object.create(null), {
    first, last, itemAt, length, after, before, slice, includes,
    contains: includes, isArray, each, forEach, join, merge,
    reverse, pluck, unique, isEmpty, splice, sort
});
